@@ -9,16 +9,18 @@ from math import isnan
 from numbers import Number
 
 import requests
-from flask import Blueprint
+from cryptoadvance.specter.util.psbt_creator import PsbtCreator
+from cryptoadvance.specter.wallet import Wallet
+from flask import Blueprint, stream_with_context
 from flask import current_app as app
 from flask import flash, jsonify, redirect, request, url_for
 from flask_babel import lazy_gettext as _
+from flask_babel import lazy_gettext
 from flask_login import current_user, login_required
 from werkzeug.wrappers import Response
 
-from cryptoadvance.specter.util.psbt_creator import PsbtCreator
-
 from ..helpers import bcur2base64
+from ..util.mnemonic import generate_mnemonic
 from ..rpc import RpcError
 from ..server_endpoints.filters import assetlabel
 from ..specter_error import SpecterError, handle_exception
@@ -33,12 +35,67 @@ logger = logging.getLogger(__name__)
 wallets_endpoint_api = Blueprint("wallets_endpoint_api", __name__)
 
 
+@wallets_endpoint_api.route("/wallets_loading/", methods=["GET", "POST"])
+@login_required
+def wallets_loading():
+    return {
+        "is_loading": app.specter.wallet_manager.is_loading,
+        "loaded_wallets": [
+            app.specter.wallet_manager.wallets[wallet].alias
+            for wallet in app.specter.wallet_manager.wallets
+        ],
+        "failed_load_wallets": [
+            wallet["alias"] for wallet in app.specter.wallet_manager.failed_load_wallets
+        ],
+    }
+
+
+@wallets_endpoint_api.route("/generatemnemonic/", methods=["GET", "POST"])
+@login_required
+def generatemnemonic():
+    return {
+        "mnemonic": generate_mnemonic(
+            strength=int(request.form["strength"]),
+            language_code=app.get_language_code(),
+        )
+    }
+
+
+@wallets_endpoint_api.route("/get_txout_set_info")
+@login_required
+@app.csrf.exempt
+def txout_set_info():
+    res = app.specter.rpc.gettxoutsetinfo()
+    return res
+
+
+@wallets_endpoint_api.route("/get_scantxoutset_status")
+@login_required
+@app.csrf.exempt
+def get_scantxoutset_status():
+    status = app.specter.rpc.scantxoutset("status", [])
+    app.specter.info["utxorescan"] = status.get("progress", None) if status else None
+    if app.specter.info["utxorescan"] is None:
+        app.specter.utxorescanwallet = None
+    return {
+        "active": app.specter.info["utxorescan"] is not None,
+        "progress": app.specter.info["utxorescan"],
+    }
+
+
 @wallets_endpoint_api.route("/fees", methods=["GET"])
 @login_required
 def fees():
     return json.dumps(get_fees(app.specter, app.config), cls=FeeEstimationResultEncoder)
 
 
+@app.route("/get_fee/<blocks>")
+@login_required
+def fees_old(blocks):
+    """Is this endpoint even used? It has been migrated from controller.py and renamed to fees_old"""
+    return app.specter.estimatesmartfee(int(blocks))
+
+
 @wallets_endpoint_api.route("/wallet/<wallet_alias>/combine/", methods=["POST"])
 @login_required
 def combine(wallet_alias):
@@ -522,7 +579,7 @@ def tx_history_csv(wallet_alias):
 
     # stream the response as the data is generated
     response = Response(
-        txlist_to_csv(wallet, txlist, app.specter, current_user, includePricesHistory),
+        stream_with_context(txlist_to_csv(wallet, txlist, includePricesHistory)),
         mimetype="text/csv",
     )
     # add a filename
@@ -554,12 +611,12 @@ def utxo_csv(wallet_alias):
         )
         # stream the response as the data is generated
         response = Response(
-            txlist_to_csv(
-                wallet,
-                txlist,
-                app.specter,
-                current_user,
-                includePricesHistory,
+            stream_with_context(
+                txlist_to_csv(
+                    wallet,
+                    txlist,
+                    includePricesHistory,
+                )
             ),
             mimetype="text/csv",
         )
@@ -660,9 +717,7 @@ def wallet_overview_txs_csv():
         includePricesHistory = request.args.get("exportPrices", "false") == "true"
         # stream the response as the data is generated
         response = Response(
-            txlist_to_csv(
-                None, txlist, app.specter, current_user, includePricesHistory
-            ),
+            stream_with_context(txlist_to_csv(None, txlist, includePricesHistory)),
             mimetype="text/csv",
         )
         # add a filename
@@ -692,9 +747,7 @@ def wallet_overview_utxo_csv():
         includePricesHistory = request.args.get("exportPrices", "false") == "true"
         # stream the response as the data is generated
         response = Response(
-            txlist_to_csv(
-                None, txlist, app.specter, current_user, includePricesHistory
-            ),
+            stream_with_context(txlist_to_csv(None, txlist, includePricesHistory)),
             mimetype="text/csv",
         )
         # add a filename
@@ -710,38 +763,44 @@ def wallet_overview_utxo_csv():
 ################## Helpers #######################
 
 # Transactions list to user-friendly CSV format
-def txlist_to_csv(wallet, _txlist, specter, current_user, includePricesHistory=False):
+def txlist_to_csv(wallet: Wallet, _txlist, includePricesHistory=False):
+    """transforms a txlist into a csv-stream. This function is not returning but yielding. As such it needs to be called
+    via wrapping it in stream_with_context
+    see https://flask.palletsprojects.com/en/1.1.x/patterns/streaming/#streaming-with-context for details
+    """
     txlist = []
     for tx in _txlist:
         if isinstance(tx["address"], list):
-            _tx = tx.copy()
+            tx_copy = tx.copy()
+            # No idea how this could be?!
             for i in range(0, len(tx["address"])):
-                _tx["address"] = tx["address"][i]
-                _tx["amount"] = tx["amount"][i]
-                txlist.append(_tx.copy())
+                tx_copy["address"] = tx["address"][i]
+                tx_copy["amount"] = tx["amount"][i]
+                txlist.append(tx_copy.copy())
         else:
             txlist.append(tx.copy())
     data = StringIO()
     w = csv.writer(data)
     # write header
     symbol = "USD"
-    if specter.price_provider.endswith("_eur"):
+    if app.specter.price_provider.endswith("_eur"):
         symbol = "EUR"
-    elif specter.price_provider.endswith("_gbp"):
+    elif app.specter.price_provider.endswith("_gbp"):
         symbol = "GBP"
     row = (
-        _("Date"),
-        _("Label"),
-        _("Category"),
-        _("Amount ({})").format(specter.unit.upper()),
-        _("Value ({})").format(symbol),
-        _("Rate (BTC/{})").format(symbol)
-        if specter.unit != "sat"
+        # For some reason (probably app-context_specific) the _ apprev of lazy_gettext does not work
+        lazy_gettext("Date"),
+        lazy_gettext("Label"),
+        lazy_gettext("Category"),
+        lazy_gettext("Amount ({})").format(app.specter.unit.upper()),
+        lazy_gettext("Value ({})").format(symbol),
+        lazy_gettext("Rate (BTC/{})").format(symbol)
+        if app.specter.unit != "sat"
         else _("Rate ({}/SAT)").format(symbol),
-        _("TxID"),
-        _("Address"),
-        _("Block Height"),
-        _("Timestamp"),
+        lazy_gettext("TxID"),
+        lazy_gettext("Address"),
+        lazy_gettext("Block Height"),
+        lazy_gettext("Timestamp"),
     )
     if not wallet:
         row = (_("Wallet"),) + row
@@ -756,7 +815,7 @@ def txlist_to_csv(wallet, _txlist, specter, current_user, includePricesHistory=F
         if not wallet:
             wallet_alias = tx.get("wallet_alias", None)
             try:
-                _wallet = specter.wallet_manager.get_by_alias(wallet_alias)
+                _wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
             except Exception as e:
                 continue
         label = _wallet.getlabel(tx["address"])
@@ -768,37 +827,42 @@ def txlist_to_csv(wallet, _txlist, specter, current_user, includePricesHistory=F
                 tx["blockheight"] = tx_raw["blockheight"]
             else:
                 tx["blockheight"] = "Unconfirmed"
-        if specter.unit == "sat":
+        if app.specter.unit == "sat":
             value = float(tx["amount"])
             tx["amount"] = round(value * 1e8)
+        amount_price = "not supported"
+        rate = "not supported"
+        if tx.get("blocktime"):
+            timestamp = tx["blocktime"]
+        else:
+            timestamp = tx["time"]
         if includePricesHistory:
-            success, rate, symbol = get_price_at(
-                specter, current_user, timestamp=tx["time"]
-            )
-        else:
-            success = False
-        if success:
-            rate = float(rate)
-            if specter.unit == "sat":
-                rate = rate / 1e8
-            amount_price = float(tx["amount"]) * rate
-            if specter.unit == "sat":
-                rate = round(1 / rate)
-        else:
-            amount_price = None
-            rate = "-"
+            try:
+                print(f"meh {app.specter.user}")
+                rate, _ = get_price_at(app.specter, timestamp=timestamp)
+                rate = float(rate)
+                if app.specter.unit == "sat":
+                    rate = rate / 1e8
+                amount_price = float(tx["amount"]) * rate
+                amount_price = round(amount_price * 100) / 100
+                if app.specter.unit == "sat":
+                    rate = round(1 / rate)
+            except SpecterError as se:
+                logger.error(se)
+                success = False
+                rate = "-"
 
         row = (
-            time.strftime("%Y-%m-%d", time.localtime(tx["time"])),
+            time.strftime("%Y-%m-%d", time.localtime(timestamp)),
             label,
             tx["category"],
-            round(tx["amount"], (0 if specter.unit == "sat" else 8)),
-            round(amount_price * 100) / 100 if amount_price is not None else "-",
+            round(tx["amount"], (0 if app.specter.unit == "sat" else 8)),
+            amount_price,
             rate,
             tx["txid"],
             tx["address"],
             tx["blockheight"],
-            tx["time"],
+            time.strftime(("%Y-%m-%d %H:%M:%S"), time.localtime(timestamp)),
         )
         if not wallet:
             row = (tx.get("wallet_alias", ""),) + row
@@ -809,7 +873,8 @@ def txlist_to_csv(wallet, _txlist, specter, current_user, includePricesHistory=F
 
 
 # Addresses list to user-friendly CSV format
-def addresses_list_to_csv(wallet):
+def addresses_list_to_csv(wallet: Wallet):
+    """Doesn't seem to be used currently?!"""
     data = StringIO()
     w = csv.writer(data)
     # write header
@@ -900,6 +965,7 @@ def wallet_addresses_list_to_csv(addresses_list):
 
 
 def process_txlist(txlist, idx=0, limit=100, search=None, sortby=None, sortdir="asc"):
+    """Prepares the txlist for the ui filtering it with the search-criterias and sorting it"""
     if search:
         search_lower = search.lower()
         txlist = [
